using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;

namespace SamsungExtract
{
    public partial class Form1 : Form
    {
        private string FirmwareFile;
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_DragEnter(object sender, DragEventArgs e)
        {
            // make sure they're actually dropping files (not text or anything else)
            if (e.Data.GetDataPresent(DataFormats.FileDrop, false) == true)
                // allow them to continue
                // (without this, the cursor stays a "NO" symbol
                e.Effect = DragDropEffects.All;
        }

        private void Form1_DragDrop(object sender, DragEventArgs e)
        {
            // transfer the filenames to a string array
            // (yes, everything to the left of the "=" can be put in the
            // foreach loop in place of "files", but this is easier to understand.)
            string[] files = (string[])e.Data.GetData(DataFormats.FileDrop);
            FirmwareFile = files[0];

            ReadFile();
        }

        private void ReadFile()
        {
            FileInfo fInfo = new FileInfo(FirmwareFile);
            FileStream firmFile = new FileStream(FirmwareFile, FileMode.Open, FileAccess.Read, FileShare.Delete);

            // Firmware file Layout
            // 4 bytes = Magic
            // 4 bytes = CRC32 for all bytes following
            // 2 bytes = Major Version
            // 2 bytes = Minor Version
            // 4 bytes = File Size
            // 2 bytes = Unknown (Kernel Loader Offset?)
            // 2 Bytes = Kernel XOR Key
            // 4 Bytes = Kernel Size
            // 2 Bytes = Unknown (FS Loader Offset?)
            // 2 Bytes = FS Key
            // 4 Bytes = FS Size
            byte[] fwHeader = new byte[32];

            byte[] fwMagic = new byte[4];
            byte[] cRC32 = new byte[4];
            short majVer;
            short minVer;
            int fwSize;
            byte[] unkKernOffset = new byte[2];
            byte[] kernKey = new byte[2];
            int kernSize;
            byte[] unkFSOffset = new byte[2];
            byte[] fsKey = new byte[2];
            int fsSize;

            string fwVersion;

            firmFile.Read(fwHeader, 0, 32);

            Array.Copy(fwHeader, 0, fwMagic, 0, 4);
            Array.Copy(fwHeader, 4, cRC32, 0, 4);
            majVer = BitConverter.ToInt16(fwHeader, 8);
            minVer = BitConverter.ToInt16(fwHeader, 10);
            fwSize = BitConverter.ToInt32(fwHeader, 12);
            Array.Copy(fwHeader, 16, unkKernOffset, 0, 2);
            Array.Copy(fwHeader, 18, kernKey, 0, 2);
            kernSize = BitConverter.ToInt32(fwHeader, 20);
            Array.Copy(fwHeader, 24, unkFSOffset, 0, 2);
            Array.Copy(fwHeader, 26, fsKey, 0, 2);
            fsSize = BitConverter.ToInt32(fwHeader, 28);

            fwVersion = majVer.ToString() + "." + minVer.ToString();

            firmFile.Position = 0;
            CRC32 ckSum = new CRC32();
            firmFile.Position = 8;
            tbCompCRC.Text = "0x" + BitConverter.ToString(ckSum.ComputeHash(firmFile)).Replace("-", "");
            
            firmFile.Close();

            string fwType = "0x" + BitConverter.ToString(fwMagic).Replace("-", "");

            switch (fwType)
            {
                case "0x5320E804":
                    tbReaderVer.Text = "SNE-60";
                    break;
                case "0x5120E804":
                    tbReaderVer.Text = "SNE-60K";
                    break;
                default:
                    tbReaderVer.Text = "0x" + BitConverter.ToString(fwMagic).Replace("-", "");
                    break;
            }

            tbFileCRC.Text = "0x" + BitConverter.ToString(cRC32).Replace("-", "");
            tbVersion.Text = fwVersion;
            tbFileSize.Text = fwSize.ToString();
            tbKernOffset.Text = "0x" + BitConverter.ToString(unkKernOffset).Replace("-", "");
            tbKernKey.Text = "0x" + BitConverter.ToString(kernKey).Replace("-", "");
            tbKernSize.Text = kernSize.ToString();
            tbFSOffset.Text = "0x" + BitConverter.ToString(unkFSOffset).Replace("-", "");
            tbFsKey.Text = "0x" + BitConverter.ToString(fsKey).Replace("-", "");
            tbFsSize.Text = fsSize.ToString();

            if (tbCompCRC.Text != tbFileCRC.Text)
            {
                tbFileCRC.BackColor = Color.Red;
            }
            else
            {
                tbFileCRC.BackColor = Color.Green;
            }

            pbDecrypt.Maximum = kernSize + fsSize;
            EncryptDecrypt(fInfo, 32, kernSize, "uImage", kernKey);
            EncryptDecrypt(fInfo, 32 + kernSize, fsSize, "FS.squashfs", fsKey);

            pbDecrypt.Value = kernSize + fsSize;
        }

        private void EncryptDecrypt(FileInfo inputFile, int cryptStart, int cryptLength, string outName, Byte[] Key)
        {
            FileStream fwStream = new FileStream(inputFile.FullName, FileMode.Open, FileAccess.Read, FileShare.None);

            byte[] message = new byte[cryptLength];
            fwStream.Position = cryptStart;
            fwStream.Read(message, 0, cryptLength);

            pbDecrypt.Step += 1;

            string outFile = inputFile.FullName + "." + outName;
            FileStream outStream = new FileStream(outFile, FileMode.Create, FileAccess.ReadWrite, FileShare.None);

            for (Int32 i = 0; i < message.Length - 1; i++)
            {
                message[i] ^= Key[i % Key.Length];
                if ((i % 1024) == 0) { pbDecrypt.Value = i; }
            }

            outStream.Write(message, 0, cryptLength);

            outStream.Close();
            fwStream.Close();
        }

        public static int Checksum(byte[] input)
        {
            // Init the cksum to 0.
            int cksum = 0;

            // Loop through the buffer and sum each dword to the total.
            for (int i = 0; i < input.Length - 4; )
            {
                cksum += BitConverter.ToInt32(input, i);
                i += 4;
            }
            return cksum;
        }

    }
}